@@ -41,7 +41,7 @@ from glanceclient import exc
 from keystoneclient.auth.identity import v2 as v2_auth
 from keystoneclient.auth.identity import v3 as v3_auth
 from keystoneclient import discover
-from keystoneclient.openstack.common.apiclient import exceptions as ks_exc
+from keystoneclient import exceptions as ks_exc
 from keystoneclient import session
 
 osprofiler_profiler = importutils.try_import("osprofiler.profiler")
@@ -214,7 +214,8 @@ class OpenStackImagesShell(object):
 
     def _find_actions(self, subparsers, actions_module):
         for attr in (a for a in dir(actions_module) if a.startswith('do_')):
-            # I prefer to be hypen-separated instead of underscores.
+            # Replace underscores with hyphens in the commands
+            # displayed to the user
             command = attr[3:].replace('_', '-')
             callback = getattr(actions_module, attr)
             desc = callback.__doc__ or ''
@@ -284,9 +285,7 @@ class OpenStackImagesShell(object):
 
         return (v2_auth_url, v3_auth_url)
 
-    def _get_keystone_session(self, **kwargs):
-        ks_session = session.Session.construct(kwargs)
-
+    def _get_keystone_auth_plugin(self, ks_session, **kwargs):
         # discover the supported keystone versions using the given auth url
         auth_url = kwargs.pop('auth_url', None)
         (v2_auth_url, v3_auth_url) = self._discover_auth_versions(
@@ -346,114 +345,110 @@ class OpenStackImagesShell(object):
                              "may not able to handle Keystone V3 credentials. "
                              "Please provide a correct Keystone V3 auth_url.")
 
-        ks_session.auth = auth
-        return ks_session
+        return auth
 
-    def _get_endpoint_and_token(self, args):
+    def _get_kwargs_to_create_auth_plugin(self, args):
+        if not args.os_username:
+            raise exc.CommandError(
+                _("You must provide a username via"
+                  " either --os-username or "
+                  "env[OS_USERNAME]"))
+
+        if not args.os_password:
+            # No password, If we've got a tty, try prompting for it
+            if hasattr(sys.stdin, 'isatty') and sys.stdin.isatty():
+                # Check for Ctl-D
+                try:
+                    args.os_password = getpass.getpass('OS Password: ')
+                except EOFError:
+                    pass
+            # No password because we didn't have a tty or the
+            # user Ctl-D when prompted.
+            if not args.os_password:
+                raise exc.CommandError(
+                    _("You must provide a password via "
+                      "either --os-password, "
+                      "env[OS_PASSWORD], "
+                      "or prompted response"))
+
+        # Validate password flow auth
+        project_info = (
+            args.os_tenant_name or args.os_tenant_id or (
+                args.os_project_name and (
+                    args.os_project_domain_name or
+                    args.os_project_domain_id
+                )
+            ) or args.os_project_id
+        )
+
+        if not project_info:
+            # tenant is deprecated in Keystone v3. Use the latest
+            # terminology instead.
+            raise exc.CommandError(
+                _("You must provide a project_id or project_name ("
+                  "with project_domain_name or project_domain_id) "
+                  "via "
+                  "  --os-project-id (env[OS_PROJECT_ID])"
+                  "  --os-project-name (env[OS_PROJECT_NAME]),"
+                  "  --os-project-domain-id "
+                  "(env[OS_PROJECT_DOMAIN_ID])"
+                  "  --os-project-domain-name "
+                  "(env[OS_PROJECT_DOMAIN_NAME])"))
+
+        if not args.os_auth_url:
+            raise exc.CommandError(
+                _("You must provide an auth url via"
+                  " either --os-auth-url or "
+                  "via env[OS_AUTH_URL]"))
+
+        kwargs = {
+            'auth_url': args.os_auth_url,
+            'username': args.os_username,
+            'user_id': args.os_user_id,
+            'user_domain_id': args.os_user_domain_id,
+            'user_domain_name': args.os_user_domain_name,
+            'password': args.os_password,
+            'tenant_name': args.os_tenant_name,
+            'tenant_id': args.os_tenant_id,
+            'project_name': args.os_project_name,
+            'project_id': args.os_project_id,
+            'project_domain_name': args.os_project_domain_name,
+            'project_domain_id': args.os_project_domain_id,
+        }
+        return kwargs
+
+    def _get_versioned_client(self, api_version, args):
         endpoint = self._get_image_url(args)
         auth_token = args.os_auth_token
 
         auth_req = (hasattr(args, 'func') and
                     utils.is_authentication_required(args.func))
-
-        if auth_req and not (endpoint and auth_token):
-            if not args.os_username:
-                raise exc.CommandError(
-                    _("You must provide a username via"
-                      " either --os-username or "
-                      "env[OS_USERNAME]"))
-
-            if not args.os_password:
-                # No password, If we've got a tty, try prompting for it
-                if hasattr(sys.stdin, 'isatty') and sys.stdin.isatty():
-                    # Check for Ctl-D
-                    try:
-                        args.os_password = getpass.getpass('OS Password: ')
-                    except EOFError:
-                        pass
-                # No password because we didn't have a tty or the
-                # user Ctl-D when prompted.
-                if not args.os_password:
-                    raise exc.CommandError(
-                        _("You must provide a password via "
-                          "either --os-password, "
-                          "env[OS_PASSWORD], "
-                          "or prompted response"))
-
-            # Validate password flow auth
-            project_info = (
-                args.os_tenant_name or args.os_tenant_id or (
-                    args.os_project_name and (
-                        args.os_project_domain_name or
-                        args.os_project_domain_id
-                    )
-                ) or args.os_project_id
-            )
-
-            if not project_info:
-                # tenant is deprecated in Keystone v3. Use the latest
-                # terminology instead.
-                raise exc.CommandError(
-                    _("You must provide a project_id or project_name ("
-                      "with project_domain_name or project_domain_id) "
-                      "via "
-                      "  --os-project-id (env[OS_PROJECT_ID])"
-                      "  --os-project-name (env[OS_PROJECT_NAME]),"
-                      "  --os-project-domain-id "
-                      "(env[OS_PROJECT_DOMAIN_ID])"
-                      "  --os-project-domain-name "
-                      "(env[OS_PROJECT_DOMAIN_NAME])"))
-
-            if not args.os_auth_url:
-                raise exc.CommandError(
-                    _("You must provide an auth url via"
-                      " either --os-auth-url or "
-                      "via env[OS_AUTH_URL]"))
-
+        if not auth_req or (endpoint and auth_token):
             kwargs = {
-                'auth_url': args.os_auth_url,
-                'username': args.os_username,
-                'user_id': args.os_user_id,
-                'user_domain_id': args.os_user_domain_id,
-                'user_domain_name': args.os_user_domain_name,
-                'password': args.os_password,
-                'tenant_name': args.os_tenant_name,
-                'tenant_id': args.os_tenant_id,
-                'project_name': args.os_project_name,
-                'project_id': args.os_project_id,
-                'project_domain_name': args.os_project_domain_name,
-                'project_domain_id': args.os_project_domain_id,
+                'token': auth_token,
                 'insecure': args.insecure,
+                'timeout': args.timeout,
                 'cacert': args.os_cacert,
                 'cert': args.os_cert,
-                'key': args.os_key
+                'key': args.os_key,
+                'ssl_compression': args.ssl_compression
             }
-            ks_session = self._get_keystone_session(**kwargs)
-            auth_token = args.os_auth_token or ks_session.get_token()
+        else:
+            ks_session = session.Session.load_from_cli_options(args)
+            auth_plugin_kwargs = self._get_kwargs_to_create_auth_plugin(args)
+            ks_session.auth = self._get_keystone_auth_plugin(
+                ks_session=ks_session, **auth_plugin_kwargs)
+            kwargs = {'session': ks_session}
 
-            endpoint_type = args.os_endpoint_type or 'public'
-            service_type = args.os_service_type or 'image'
-            endpoint = args.os_image_url or ks_session.get_endpoint(
-                service_type=service_type,
-                interface=endpoint_type,
-                region_name=args.os_region_name)
+            if endpoint is None:
+                endpoint_type = args.os_endpoint_type or 'public'
+                service_type = args.os_service_type or 'image'
+                endpoint = ks_session.get_endpoint(
+                    service_type=service_type,
+                    interface=endpoint_type,
+                    region_name=args.os_region_name)
 
-        return endpoint, auth_token
-
-    def _get_versioned_client(self, api_version, args):
-        endpoint, token = self._get_endpoint_and_token(args)
-
-        kwargs = {
-            'token': token,
-            'insecure': args.insecure,
-            'timeout': args.timeout,
-            'cacert': args.os_cacert,
-            'cert': args.os_cert,
-            'key': args.os_key,
-            'ssl_compression': args.ssl_compression
-        }
-        client = glanceclient.Client(api_version, endpoint, **kwargs)
-        return client
+        return glanceclient.Client(api_version, endpoint, **kwargs)
 
     def _cache_schemas(self, options, client, home_dir='~/.glanceclient'):
         homedir = os.path.expanduser(home_dir)
@@ -467,7 +462,7 @@ class OpenStackImagesShell(object):
             except OSError as e:
                 # This avoids glanceclient to crash if it can't write to
                 # ~/.glanceclient, which may happen on some env (for me,
-                # it happens in Jenkins, as Glanceclient can't write to
+                # it happens in Jenkins, as glanceclient can't write to
                 # /var/lib/jenkins).
                 msg = '%s' % e
                 print(encodeutils.safe_decode(msg), file=sys.stderr)
@@ -499,18 +494,12 @@ class OpenStackImagesShell(object):
             try:
                 return self.get_subcommand_parser(api_version)
             except ImportError as e:
-                if options.debug:
-                    traceback.print_exc()
                 if not str(e):
                     # Add a generic import error message if the raised
                     # ImportError has none.
                     raise ImportError('Unable to import module. Re-run '
                                       'with --debug for more info.')
                 raise
-            except Exception:
-                if options.debug:
-                    traceback.print_exc()
-                raise
 
         # Parse args once to find version
 
@@ -529,7 +518,7 @@ class OpenStackImagesShell(object):
             endpoint = self._get_image_url(options)
             endpoint, url_version = utils.strip_version(endpoint)
         except ValueError:
-            # NOTE(flaper87): ValueError is raised if no endpoint is povided
+            # NOTE(flaper87): ValueError is raised if no endpoint is provided
             url_version = None
 
         # build available subcommands based on version
@@ -545,6 +534,7 @@ class OpenStackImagesShell(object):
         # Handle top-level --help/-h before attempting to parse
         # a command off the command line
         if options.help or not argv:
+            parser = _get_subparser(api_version)
             self.do_help(options, parser=parser)
             return 0
 
@@ -591,6 +581,12 @@ class OpenStackImagesShell(object):
         if not args.os_password and options.os_password:
             args.os_password = options.os_password
 
+        if args.debug:
+            # Set up the root logger to debug so that the submodules can
+            # print debug messages
+            logging.basicConfig(level=logging.DEBUG)
+            # for iso8601 < 0.1.11
+            logging.getLogger('iso8601').setLevel(logging.WARNING)
         LOG = logging.getLogger('glanceclient')
         LOG.addHandler(logging.StreamHandler())
         LOG.setLevel(logging.DEBUG if args.debug else logging.INFO)
@@ -605,12 +601,6 @@ class OpenStackImagesShell(object):
             args.func(client, args)
         except exc.Unauthorized:
             raise exc.CommandError("Invalid OpenStack Identity credentials.")
-        except Exception:
-            # NOTE(kragniz) Print any exceptions raised to stderr if the
-            # --debug flag is set
-            if args.debug:
-                traceback.print_exc()
-            raise
         finally:
             if profile:
                 trace_id = osprofiler_profiler.get().get_base_id()
@@ -630,14 +620,24 @@ class OpenStackImagesShell(object):
             else:
                 raise exc.CommandError("'%s' is not a valid subcommand" %
                                        args.command)
-            command = ' ' + command
         else:
             parser.print_help()
 
         if not args.os_image_api_version or args.os_image_api_version == '2':
-            print()
-            print(("Run `glance --os-image-api-version 1 help%s` "
-                   "for v1 help") % command)
+            # NOTE(NiallBunting) This currently assumes that the only versions
+            # are one and two.
+            try:
+                if command is None:
+                    print("\nRun `glance --os-image-api-version 1 help`"
+                          " for v1 help")
+                else:
+                    self.get_subcommand_parser(1)
+                    if command in self.subcommands:
+                        command = ' ' + command
+                        print(("\nRun `glance --os-image-api-version 1 help%s`"
+                               " for v1 help") % (command or ''))
+            except ImportError:
+                pass
 
     def do_bash_completion(self, _args):
         """Prints arguments for bash_completion.
@@ -671,4 +671,6 @@ def main():
     except KeyboardInterrupt:
         utils.exit('... terminating glance client', exit_code=130)
     except Exception as e:
+        if utils.debug_enabled(argv) is True:
+            traceback.print_exc()
         utils.exit(encodeutils.exception_to_unicode(e))
